
require_relative "gameobject"
require_relative "lib/nonoengine"

class Corpse < PhysicObject
  attr_accessor :timer

  def initialize(from, blood)
    super(MAT_CORPSE)
    @blood = blood
    if blood
      @picture = TILE_BLOOD
      @timer = 60 * 5 + rand(60 * 5)
      warp(from.x + rand(OBJSIZE / 2) - OBJSIZE / 4,
           from.y + rand(OBJSIZE / 2) - OBJSIZE / 4)
      @layer = LAYER_BLOOD
      @size = OBJSIZE * 0.2
    else
      @picture = TILE_CORPSE1 + rand(2)
      @timer = 60 * 10
      warp(from.x, from.y)
      @layer = LAYER_GORE
      @size = OBJSIZE * 0.75
    end
  end

  def update
    @timer -= 1
    @dead = true if @timer < 0
  end

  def draw
    Engine.layer = @layer
    n = 3 * 60
    if @timer < n
      color = Engine::Colors.new(150, 150, 150, 255)
    else
      color = Engine::Colors::WHITE
    end
    RES::All.draw_frame(@picture, x, y, @size / SPRITESIZE, @size / SPRITESIZE, 0, color)
    Engine.layer = LAYER_UNITS
  end

  def collide_walls(wall)
    true
  end

  def collide_door(door)
    true
  end

  def init_form
    physics.add_circle_shape @body, OBJSIZE / 8, 0, 0
  end

  def dontcollide_enemy
  end

  def dontcollide_golem
  end

  def dontcollide_bullet
  end

  def dontcollide_corpse
  end

  def collide_player(player)
    if @layer == LAYER_BLOOD && @timer > 3 * 60 && player.passive && player.passive.class == BloodMagic
      player.passive.get_blood
      @timer = 1
    end

    false
  end
end

HIT_TIME = 30

class BloodHit < GameObject
  attr_accessor :x, :y, :angle, :timer

  def initialize(source, fromx, fromy)
    @timer = HIT_TIME + rand(10)
    v = -(vec2(fromx, fromy) - source.pos).normalize * 48
    @x = source.x + v.x
    @y = source.y + v.y
    @angle = Math.atan2(fromx - source.x, fromy - source.y) * 180 / Math::PI + rand(20) - 10
  end

  def update
    @timer -= 1
    if @timer < 0
      @dead = true
      if rand < BLOOD_LEVEL
        $app.objects << Corpse.new(self, true)
      end
    end
  end

  def draw
    n = ((HIT_TIME - @timer) * 1.0 / HIT_TIME * 16).to_i
    RES::Blood_hit.draw_frame(n, x, y, 0.5, 0.5, angle)
  end
end

def circle_rand(x, y, r)
  loop do
    ax = x + rand(2 * r) - r
    ay = y + rand(2 * r) - r
    return [ax, ay] if (ax - x) ** 2 + (ay - y) ** 2 < r * r
  end
end

class JustBone < GameObject
  attr_accessor :x, :y

  def initialize(x, y)
    @id = rand(6)
    @x, @y = circle_rand(x, y, 300)
  end

  def draw
    Engine.layer = LAYER_GORE
    RES::Bones.draw_frame(@id, @x, @y, 2, 2)
    Engine.layer = LAYER_UNITS
  end
end

def massacre(x, y)
  200.times do
    bone = JustBone.new(x, y)
    $app.objects << bone
    blood = Corpse.new(bone, true)
    blood.timer = 60 * 19 * 60
    $app.objects << blood
  end
end
