require "fiddle"
require "rbconfig"

module Bearlib
  module Raw
    OS ||=
      (host_os = RbConfig::CONFIG["host_os"]
        case host_os
      when /mswin|msys|mingw|cygwin|bccwin|wince|emc/
        :windows
      when /darwin|mac os/
        :macosx
      when /linux/
        :linux
      when /solaris|bsd/
        :unix
      else
        raise RuntimeError, "Unknown OS: #{host_os.inspect}"
      end)

    case OS
    when /linux/
      Libfovname = "./libBearLibFOV.so"
      Libmapname = "./libBearLibMap.so"
      Libmgname = "./libBearLibMG.so"
    when /macosx/
      Libfovname = "./libBearLibFOV.dylib"
      Libmapname = "./libBearLibMap.dylib"
      Libmgname = "./libBearLibMG.dylib"
    when /windows/
      Libfovname = "BearLibFOV.dll"
      Libmapname = "BearLibMap.dll"
      Libmgname = "BearLibMG.dll"
    else
      raise RuntimeError, "Unsupported OS: #{OS}"
    end

    Libfov = Fiddle.dlopen(Libfovname)
    Libmap = Libfov
    Libmg = Fiddle.dlopen(Libmgname)
    Ptr = Fiddle::Pointer
  end
end
