require "fiddle"
require "rbconfig"
require_relative "bearlib"

module Bearlib
  module Raw
    # Raw API entries
    Generate = Fiddle::Function.new(Libmg["mg_generate"], [Fiddle::TYPE_INT, Fiddle::TYPE_INT, Fiddle::TYPE_INT, Fiddle::TYPE_INT, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP], Fiddle::TYPE_VOID)
    GenerateCB = Fiddle::Function.new(Libmg["mg_generate_cb"], [Fiddle::TYPE_INT, Fiddle::TYPE_INT, Fiddle::TYPE_INT, Fiddle::TYPE_INT, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP], Fiddle::TYPE_VOID)
    ParamsCreate = Fiddle::Function.new(Libmg["mg_params_create"], [Fiddle::TYPE_INT], Fiddle::TYPE_VOIDP)
    ParamsDelete = Fiddle::Function.new(Libmg["mg_params_delete"], [Fiddle::TYPE_VOIDP], Fiddle::TYPE_VOID)
    ParamsSet = Fiddle::Function.new(Libmg["mg_params_set"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_INT], Fiddle::TYPE_VOID)
    ParamsSetF = Fiddle::Function.new(Libmg["mg_params_setf"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_FLOAT], Fiddle::TYPE_VOID)
    ParamsGet = Fiddle::Function.new(Libmg["mg_params_get"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP], Fiddle::TYPE_INT)
    ParamsGetF = Fiddle::Function.new(Libmg["mg_params_getf"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP], Fiddle::TYPE_FLOAT)
    ParamsSetStr = Fiddle::Function.new(Libmg["mg_params_setstr"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP], Fiddle::TYPE_VOID)
    RoomsDataCreate = Fiddle::Function.new(Libmg["mg_roomsdata_create"], [Fiddle::TYPE_INT], Fiddle::TYPE_VOIDP)
    RoomsDataDelete = Fiddle::Function.new(Libmg["mg_roomsdata_delete"], [Fiddle::TYPE_VOIDP], Fiddle::TYPE_VOID)
    RoomsDataCount = Fiddle::Function.new(Libmg["mg_roomsdata_count"], [Fiddle::TYPE_VOIDP], Fiddle::TYPE_INT)
    RoomsDataPosition = Fiddle::Function.new(Libmg["mg_roomsdata_position"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_INT, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP, Fiddle::TYPE_VOIDP], Fiddle::TYPE_VOID)
    RoomsDataLinkCount = Fiddle::Function.new(Libmg["mg_roomsdata_linkscount"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_INT], Fiddle::TYPE_INT)
    RoomsDataGetLink = Fiddle::Function.new(Libmg["mg_roomsdata_getlink"], [Fiddle::TYPE_VOIDP, Fiddle::TYPE_INT, Fiddle::TYPE_INT], Fiddle::TYPE_INT)
  end

  module Constants
    G_ANT_NEST = 1
    G_CAVES = 2
    G_VILLAGE = 3
    G_LAKES = 4
    G_LAKES2 = 5
    G_TOWER = 6
    G_HIVE = 7
    G_CITY = 8
    G_MOUNTAIN = 9
    G_FOREST = 10
    G_SWAMP = 11
    G_RIFT = 12
    G_TUNDRA = 13
    G_BROKEN_CITY = 14
    G_BROKEN_VILLAGE = 15
    G_MAZE = 16
    G_CASTLE = 17
    G_WILDERNESS = 18
    G_NICE_DUNGEON = 19

    TILE_CAVE_WALL = 0
    TILE_GROUND = 1
    TILE_WATER = 2
    TILE_TREE = 3
    TILE_MOUNTAIN = 4
    TILE_ROAD = 6
    TILE_HOUSE_WALL = 7
    TILE_HOUSE_FLOOR = 8
    TILE_GRASS = 9
    TILE_EMPTY = 10
    TILE_CORRIDOR = 11
    TILE_SMALLROOM = 12
    TILE_BIGROOM = 13
    TILE_ROOMWALL = 14
    TILE_DOOR = 15
  end

  include Constants

  def self.passable?(x)
    [Bearlib::TILE_GROUND,
     Bearlib::TILE_WATER,
     Bearlib::TILE_ROAD,
     Bearlib::TILE_HOUSE_FLOOR,
     Bearlib::TILE_GRASS,

     Bearlib::TILE_CORRIDOR,
     Bearlib::TILE_SMALLROOM,
     Bearlib::TILE_BIGROOM,
     Bearlib::TILE_DOOR].include? x
  end

  # Wrappers

  class Layer
    def generate(typ, seed)
      Raw::Generate.call(@owner.handle, @id, typ, seed, nil, nil)
    end
  end
end
