require_relative "dance"

class DancerDanceSkill < DanceSkill
  def initialize(*args)
    super
    @start = rand * 2 * Math::PI
    @start_sign = @stats[:sign]
  end

  def ignore(enemy)
    enemy == @target
  end

  def update
    if @active
      angle = @start + @start_sign * 8 * Math::PI * @delay / @stats[:delay]
      @owner.body.apply_impulse(vec2(Math.cos(angle), Math.sin(angle)) * @stats[:force], vec2(0, 0))
      @target.body.apply_impulse(-vec2(Math.cos(angle), Math.sin(angle)) * @stats[:force], vec2(0, 0)) if @target
      @delay += 1

      other_skill.delay -= 0.75 if other_skill.delay > 0

      deactivate if @stats[:duet] && @target.dead
    else
      @delay -= 0.25 if @delay > 0
    end
  end

  def touch_ok
    @owner.last_touch && !@owner.last_touch.dead && (@owner.last_touch.pos - @owner.pos).length < 2 * OBJSIZE
  end

  def progress
    return 0.0 if @stats[:duet] && !@active && !touch_ok
    super
  end

  def activate
    if other_skill.active
      other_skill.deactivate
    end

    if @stats[:duet]
      return unless @owner.last_touch
      @target = @owner.last_touch
      return if @target.dead
      return if (@target.pos - @owner.pos).length > 2 * OBJSIZE

      @leash = CP::Constraint::PivotJoint.new(@owner.body, @target.body, CP::ZERO_VEC_2, CP::ZERO_VEC_2)
      @leash.max_bias = @stats[:leash_speed]
      @leash.max_force = @stats[:leash_force]
      $app.physics.add_constraint(@leash, @owner.body)

      @stats[:ignore] = @target
    end

    @active = true
    @owner.dancing = @stats
  end

  def deactivate
    @active = false
    @owner.dancing = nil

    if @stats[:duet]
      $app.physics.remove_constraint(@leash, @owner.body)
      @target = nil
    end
  end

  def apply(dx, dy, btn_state)
    if @delay > @stats[:delay] || btn_state == :clicked
      deactivate
      return
    end

    activate if !@active && @delay < @stats[:delay] * 0.9
  end
end

class DancePain < Skill
  def update
    if @owner.pain > 0 && @owner.left_skill.active || @owner.right_skill.active
      @owner.pain = [@owner.pain - @stats[:power], 0.0].max
    end
  end
end

DANCER_SOLO = {
  desc: "Вращение соло",
  class: DancerDanceSkill,
  delay: 120,
  damage: 4,
  force: 400.0,
  sign: -1,
}

DANCER_DUET = {
  desc: "Вращение с партнером",
  class: DancerDanceSkill,
  delay: 120,
  damage: 4,
  force: 400.0,
  sign: 1,
  duet: true,
  leash_speed: 1000.0,
  leash_force: 10000.0,
}

DANCE_DESC = <<me
танцуя забывает о боли
  вращение в обратную сторону восстанавливает баланс
me

DANCER =
  {
    name: "Танцор с мечами",
    desc: DANCE_DESC,
    material: MAT_PLAYER,
    picture: 13 * 8 + 6,

    left_skill: DANCER_SOLO,
    right_skill: DANCER_DUET,
    passive: {
      class: DancePain,
      power: 0.1,
    },
  }
