require_relative "../bullet"
require_relative "../graph_consts"
require_relative "../lib/nonoengine"

class Skill
  attr_accessor :owner
  attr_accessor :stats

  def initialize(owner, stats)
    @stats = stats
    @owner = owner
  end

  def apply(dx, dy, btn_state)
  end

  def update
  end

  def progress
    1.0
  end

  def other_skill
    @owner.left_skill == self ? @owner.right_skill : @owner.left_skill
  end

  def draw_hud(is_left)
    x = is_left ? 100 : SCREENX - 100
    y = SCREENY - 100
    pic = @stats[:hud] || (@owner.klass[:picture] + (is_left ? -2 : -1) * 8)
    RES::All.draw_frame(pic, x, y, 100 / SPRITESIZE, 100 / SPRITESIZE)
    show_progress(x, y, 50, progress.clamp(0, 1))
  end
end

class ShootSkill < Skill
  attr_accessor :delay

  def initialize(*args)
    super
    @delay = 0
  end

  def maxdistance
    @stats[:speed] * PHYS_DT * SUBSTEPS * @stats[:maxrange] + OBJSIZE
  end

  def progress
    1.0 - 1.0 * @delay / @stats[:delay]
  end

  def update
    return if @delay <= 0
    @delay = 0 if @delay < 1.1
    if @stats[:linked]
      return if other_skill.delay > 1 && other_skill.delay < @delay
    end
    @delay -= 1
  end

  def apply(dx, dy, btn_state)
    return if @delay > 0
    @delay = @stats[:delay]
    spread = @stats[:spread]
    @stats[:count].times do
      angle = Math.atan2(dy, dx)
      angle += rand * spread - spread / 2.0
      angle += @stats[:offset] if @stats[:offset]
      spd = vec2(Math.cos(angle), Math.sin(angle)).normalize * @stats[:speed] * (1 + rand / 10) #+ @owner.speed
      shot = Bullet.new(@owner.x, @owner.y, spd.x, spd.y, @owner.side, @stats)
      $app.objects << shot
    end
    @stats[:sound].play if @stats[:sound]
    @owner.hidden = false if @owner.class == Player
  end
end

N_CIRCLE = 32

def show_progress(x, y, r, value)
  xp = x
  yp = y
  value = 1 - value
  (value * (N_CIRCLE + 1)).to_i.times do |i|
    a = -Math::PI / 2 - 2 * Math::PI / N_CIRCLE * i
    xn = x + Math.cos(a) * r
    yn = y + Math.sin(a) * r
    triangle2(xp, yp, xn, yn, x, y, 0x00000080)
    xp = xn
    yp = yn
  end
end

def show_revertprogress(x, y, r, value)
  xp = x
  yp = y
  (value * (N_CIRCLE + 1)).to_i.times do |i|
    a = -Math::PI / 2 + 2 * Math::PI / N_CIRCLE * i
    xn = x + Math.cos(a) * r
    yn = y + Math.sin(a) * r
    triangle2(xp, yp, xn, yn, x, y, 0x00000040)
    xp = xn
    yp = yn
  end
end

class Empathy < Skill
  def react(attack)
    @owner.pain += attack[:damage] * @stats[:level]
    @owner.pain = 0 if @owner.pain < 0
  end
end

# require_relative "./test"
require_relative "./monster"
require_relative "./warrior"
require_relative "./barbarian"
require_relative "./gunslinger"
require_relative "./battlemage"
require_relative "./healer"
require_relative "./hunter"
require_relative "./dancer"
require_relative "./warlock"
require_relative "./bloodmage"
require_relative "./lastone"

ALL_CLASSES = [SLINGER, BARBARIAN, BATTLEMAGE, HEALER, HUNTER, WARRIOR, DANCER, WARLOCK, BLOODMAGE]
#ALL_CLASSES = [FINALONE]
