require_relative "./lib/nonoengine"
require_relative "./resources"

class Story
  attr_accessor :storypoint, :stage, :stage_timer, :level

  def initialize
    $game.story = self
    @town = GameLevel.new(TownGenerator.new)
    @portal_town = GameLevel.new(PortalTownGenerator.new)
    @castle = GameLevel.new(DungeonGenerator.new)
    @storypoint = :at_home
    player_died
  end

  def draw
    case @stage
    when :splash
      show_splash @stage_timer
    when :splash2
      show_splash2 @stage_timer
    when :splash3
      show_splash3 @stage_timer
    when :help
      show_help(@klass, @stage_timer)
    when :stats
      show_stats @stage_timer
    end
  end

  def game_over
    @stage = :stats
    @stage_timer = 0
  end

  def action
    @stage == :action
  end

  def player_died
    @stage = :splash
    @stage_timer = 0
    case @storypoint
    when :at_home
      @level = GameLevel.new(HouseGenerator.new)
    when :at_street
      @level = @town
    when :at_castle
      @level = @town
    when :at_portal
      @level = @portal_town
      if @died_at_portal
        @stage = :stats
        @demon_dead = true
        #@level = nil
        return
      else
        @died_at_portal = true
        @stage = :splash2
      end
    end
    @level.activate
    if @level == @portal_town
      @klass = FINALONE
    else
      @klass = ALL_CLASSES.sample
    end
    $app.new_player(@klass)
  end

  def next_level
    oldguy = @level.player
    case @storypoint
    when :at_home
      @storypoint = :at_street
      @level = @town
    when :at_street
      @storypoint = :at_castle
      @level = @castle
    when :at_castle
      if level == @town
        @storypoint = :at_castle
        @level = @castle
      else
        @storypoint = :at_portal
        @level = @portal_town
      end
    end
    RES::Music_2.music(25) if @level == @castle
    @level.activate
    @level.existing_player(oldguy)
    oldguy.dead = true
  end

  def update
    @stage_timer += Engine[Params::DeltaTime]
    key_pressed = Engine::Key[Keys::KEY_SPACE] == :pressed
    case @stage
    when :splash
      if key_pressed
        if @stage_timer < 5000
          @stage_timer = 5000
        else
          @stage = :help
          @stage_timer = 0
        end
      end
    when :splash2
      if key_pressed
        @stage = :splash3
        @stage_timer = 0
      end
    when :splash3
      if key_pressed && @stage_timer > 2000
        @stage = :action
        @stage_timer = 0
      end
    when :help
      if key_pressed
        @stage = :action
        RES::Music_1.music(25)
      end
    end
  end
end

POINT_NAMES = {
  at_home: "спящим в своей кровати",
  at_street: "в бою, на пороге твоего дома",
  at_castle: "в резне перед замком",
}

THE_TEXT = <<me
  Буду краток. Ты мертв.
  Вы все мертвы, весь город. Демоны убили вас ночью. 
  Тебя - %s
  А теперь у тебя есть возможность отомстить.
  Нет, я не из них, я их враг. Да и какое тебе дело до моих мотивов.
  Главное что я даю тебе второй шанс.
  Ты ведь хочешь этого? Умереть еще раз, но забрав врага с собой.
  Мои чары развеются с первыми лучами солнца, но до этого момента я буду
   пробуждать тех из вас кто готов бороться.

  Больше некому. Встань и дерись.

  <Пробел чтобы продолжить>
me

THE_TEXT_2 = <<me
  Убит? 
  Наконец-то!
  Немедленно закройте портал, хватит с нас этих мертвецов.
  Надеюсь, больше ни одного не осталось по эту сторону портала.







  
  <Пробел чтобы продолжить>
me

THE_TEXT_3 = <<me
  Еще один остался. 
  
  Наконец я добрался до вас.

  Держитесь, гады.

  Грррххххх!





  <Пробел чтобы продолжить>
me

def show_splash2(time)
  RES::All.draw_frame(TILE_DEMON1, SCREENX / 2 - 200, SCREENY / 2 + 100, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  RES::All.draw_frame(TILE_DEMON1, SCREENX / 2 + 100, SCREENY / 2 + 0, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  RES::All.draw_frame(TILE_DEMON1, SCREENX / 2 - 400, SCREENY / 2 + 40, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  RES::All.draw_frame(TILE_DEMON1, SCREENX / 2 + 360, SCREENY / 2 + 60, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  RES::All.draw_frame(TILE_DEMON1, SCREENX / 2, SCREENY / 2 + 70, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  Engine.rect2(50, SCREENY - 350, SCREENX - 70, 340, true, Engine::Colors.new(50, 0, 0, 255))
  RES::Vera.draw_text(THE_TEXT_2, 70, SCREENY - 340)
end

def show_splash3(time)
  if time > 1000
    RES::All.draw_frame(TILE_DEMON2, SCREENX / 2, SCREENY / 2 + 70, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  end
  if time > 2000
    Engine.rect2(50, SCREENY - 350, SCREENX - 70, 340, true, Engine::Colors.new(50, 0, 0, 255))
    RES::Vera.draw_text(THE_TEXT_3, 70, SCREENY - 340)
  end
end

def show_splash(time)
  RES::All.draw_frame(TILE_DEMON2, SCREENX / 2, SCREENY / 2 + 70, SCREENY / SPRITESIZE, SCREENY / SPRITESIZE)
  eye = time / 20
  eye = 3000 / 20 + (time - 3000) if time > 3000
  Engine.rect2(0, 0, SCREENX, SCREENY / 2 - eye / 2 - 125, true, Engine::Colors.new(50, 0, 0, 255))
  Engine.rect2(0, SCREENY / 2 + eye / 2 - 125, SCREENX, SCREENY, true, Engine::Colors.new(50, 0, 0, 255))
  if time > 3000 + 2000
    Engine.rect2(50, SCREENY - 350, SCREENX - 70, 340, true, Engine::Colors.new(50, 0, 0, 255))
    RES::Vera.draw_text(sprintf(THE_TEXT, POINT_NAMES[@storypoint]), 70, SCREENY - 340)
  else
    RES::Vera.draw_text("<Пробел чтобы продолжить>", SCREENX / 2 - 120, SCREENY - 50) if time > 0
  end
end

def show_help(klass, time)
  Engine.rect2(0, 0, SCREENX, SCREENY, true, Engine::Colors.new(0, 0, 0, 100))
  RES::Vera.draw_text("Персонаж: " + klass[:name], SCREENX / 2 - 200, 250)
  RES::Vera.draw_text("Особенность: " + klass[:desc], SCREENX / 2 - 190, 275)
  RES::Vera.draw_text("левая кнопка мыши:", 20, SCREENY - 200)
  RES::Vera.draw_text(klass[:left_skill][:desc], 20, SCREENY - 150)
  RES::Vera.draw_text("правая кнопка мыши:", SCREENX - 250, SCREENY - 200)
  RES::Vera.draw_text(klass[:right_skill][:desc], SCREENX - 250, SCREENY - 150)
  RES::Vera.draw_text("Индикатор боли и урона", SCREENX / 2 - 120, SCREENY - 100) unless klass[:passive] && klass[:passive][:class] == NoPain
  RES::Vera.draw_text("<Пробел чтобы продолжить>", SCREENX / 2 - 120, SCREENY - 30) if (time / 1000) % 2 == 1
end

def show_stats(time)
  RES::Gui::Music.music
  Engine.rect2(0, 0, SCREENX, SCREENY, true, Engine::Colors.new(50, 0, 0, 255))
  camera(0, 0)
  Engine.panel(10, 0, 0, 100, 500, 500, Align::CENTER, Align::NONE)
  RES::Gui::Back.show(10, 0, 0, 800, 500, Align::CENTER, Align::CENTER)
  return if time < 1000
  if @storypoint == :at_portal
    if @demon_dead
      result1 = "Месть красноглазого подошла к концу"
      result2 = "Он был убит в бою, добравшись наконец"
      result3 = "до родного мира демонов"
    elsif @level.player.klass == FINALONE
      result1 = "Ожидание красноглазого подошло к концу"
      result2 = "Он смог добраться до родного мира демонов"
      result3 = "Вероятно, рано или поздно его убьют, но кто знает..."
    else
      result1 = "Восход должен был наступить, но в этом мире нет Солнца"
      result2 = "Вы в родном мире демонов"
      result3 = "Незавидная ситуация, но вы все еще живы"
    end
  else
    result1 = "Солнце взошло и чары красноглазого развеялись"
    result2 = "Перед смертью вы успеваете"
    result3 = "насладиться видом трупов врагов вокруг вас"
  end

  result4 = "Убито прислужников: #{$game.statistics[nil]}"
  result5 = "Убито демонов: #{$game.statistics[true]}"

  dy = 30
  dt0 = 1000
  dt = 700
  return if time < dt0
  RES::Gui::Empty.show(1, 10, 100 + dy, 800, dy, Align::CENTER, Align::NONE, result1, RES::Vera)
  return if time < dt0 + dt
  RES::Gui::Empty.show(1, 10, 100 + 2 * dy, 800, dy, Align::CENTER, Align::NONE, result2, RES::Vera)
  return if time < dt0 + 2 * dt
  RES::Gui::Empty.show(1, 10, 100 + 3 * dy, 800, dy, Align::CENTER, Align::NONE, result3, RES::Vera)
  return if time < dt0 + 3 * dt
  RES::Gui::Empty.show(1, 10, 100 + 6 * dy, 800, dy, Align::CENTER, Align::NONE, result4, RES::Vera)
  return if time < dt0 + 4 * dt
  RES::Gui::Empty.show(1, 10, 100 + 8 * dy, 800, dy, Align::CENTER, Align::NONE, result5, RES::Vera)
  return if time < dt0 + 6 * dt
  RES::Gui::Empty.show(1, 10, 100 + 10 * dy, 800, dy, Align::CENTER, Align::NONE, "ИГРА ОКОНЧЕНА", RES::Vera)
end
